@@ -1,4 +1,4 @@
-/* $Id: timer.c 1897 2007-08-25 04:57:38Z aturner $ */
+/* $Id: timer.c 2006 2008-05-01 04:46:55Z aturner $ */
 
 /*
  * Copyright (c) 2001-2007 Aaron Turner.
@@ -56,20 +56,16 @@ timerdiv(struct timeval *tvp, float div)
     tvp->tv_usec = interval - (tvp->tv_sec * 1000000);
 }
 
-/*
- * converts a float to a timeval structure
- * converted to a #define
-void
-float2timer(float time, struct timeval *tvp)
+/* Divide tvs by div, storing the result in tvs */
+void timesdiv(struct timespec *tvs, float div)
 {
-    float n;
-
-    n = time;
-
-    tvp->tv_sec = n;
-
-    n -= tvp->tv_sec;
-    tvp->tv_usec = n * 100000;
-
+    double interval;
+    
+    if (div == 0 || div == 1)
+        return;
+        
+    interval = ((double)tvs->tv_sec * 1000000000 + tvs->tv_nsec) / (double)div;
+    tvs->tv_sec = interval / (int)1000000000;
+    tvs->tv_nsec = interval - (tvs->tv_nsec * 1000000000);
 }
- */
\ No newline at end of file
+
