@@ -161,7 +161,8 @@ enum tcpprep_mode {
 #define MAC_STR(x) x[0], x[1], x[2], x[3], x[4], x[5]
 
 /* struct timeval print structs */
-#define TIMEVAL_FORMAT "%lu.%06lu"
+#define TIMEVAL_FORMAT "%lus %luusec"
+#define TIMESPEC_FORMAT "%lus %lunsec"
 
 /* force a word or half-word swap on both Big and Little endian systems */
 #ifndef SWAPLONG
@@ -203,4 +204,73 @@ enum tcpprep_mode {
 #endif
 #endif
 
+#ifndef HAVE_UINT8_T
+typedef u_int8_t uint8_t
+typedef u_int16_t uint16_t
+typedef u_int32_t uint32_t
+#endif
+
+/* Support for flexible arrays. */
+#undef __flexarr
+#if defined(__GNUC__) && ((__GNUC__ > 2) || (__GNUC__ == 2 && __GNUC_MINOR__ >= 97))
+/* GCC 2.97 supports C99 flexible array members.  */
+# define __flexarr      []
+#else
+# ifdef __GNUC__
+#  define __flexarr     [0]
+# else
+#  if defined(__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
+#   define __flexarr    []
+#  elif defined(_WIN32)
+/* MS VC++ */
+#   define __flexarr    []
+#  else
+/* Some other non-C99 compiler. Approximate with [1]. */
+#   define __flexarr    [1]
+#  endif
+# endif
+#endif
+
+
+/* Time converters */
+#define SEC_TO_MILLISEC(x) (x * 1000)
+#define SEC_TO_MICROSEC(x) (x * 1000000)
+#define SEC_TO_NANOSEC(x)  ((u_int64_t)x * 1000000000)
+
+#define MILLISEC_TO_SEC(x) (x / 1000)
+#define MICROSEC_TO_SEC(x) (x / 1000000)
+#define NANOSEC_TO_SEC(x)  ((u_int64_t)x / 1000000000)
+
+#define TIMEVAL_TO_MILLISEC(x)  (((x)->tv_sec * 1000) + ((x)->tv_usec / 1000))
+#define TIMEVAL_TO_MICROSEC(x)  (((x)->tv_sec * 1000000) + (x)->tv_usec)
+#define TIMEVAL_TO_NANOSEC(x)   ((u_int64_t)((x)->tv_sec * 1000000000) + ((u_int64_t)(x)->tv_usec * 1000))
+
+#define MILLISEC_TO_TIMEVAL(x, tv)          \
+    do {                                    \
+        (tv)->tv_sec = (x) / 1000;          \
+        (tv)->tv_usec = (x * 1000) - ((tv)->tv_sec * 1000000);   \
+    } while(0)
+    
+#define MICROSEC_TO_TIMEVAL(x, tv)          \
+    do {                                    \
+        (tv)->tv_sec = (x) / 1000000;       \
+        (tv)->tv_usec = (x) - ((tv)->tv_sec * 1000000);   \
+    } while(0)
+
+#define NANOSEC_TO_TIMEVAL(x, tv)           \
+    do {                                    \
+        (tv)->tv_sec =  (x) / 1000000000;   \
+        (tv)->tv_usec = ((x) % 1000000000) / 1000);   \
+    } while(0)
+
+#define NANOSEC_TO_TIMESPEC(x, ts)          \
+    do {                                    \
+        (ts)->tv_sec = (x) / 1000000000;    \
+        (ts)->tv_nsec = (x) % 1000000000;   \
+    } while(0)
+
+#define TIMESPEC_TO_MILLISEC(x)  (((x)->tv_sec * 1000) + ((x)->tv_nsec / 1000000))
+#define TIMESPEC_TO_MICROSEC(x)  (((x)->tv_sec * 1000000) + (x)->tv_nsec / 1000)
+#define TIMESPEC_TO_NANOSEC(x) ((u_int64_t)((x)->tv_sec * 1000000000) + ((u_int64_t)(x)->tv_nsec))
+
 #endif /* DEFINES */
