@@ -1,4 +1,4 @@
-/* $Id: tcpbridge.c 1842 2007-04-26 04:34:11Z aturner $ */
+/* $Id: tcpbridge.c 2027 2008-05-15 16:54:58Z aturner $ */
 
 /*
  * Copyright (c) 2004-2005 Aaron Turner.
@@ -122,7 +122,7 @@ main(int argc, char *argv[])
     sendpacket_close(options.sp1);
     pcap_close(options.listen1);
 
-    if (! options.unidir) {
+    if (options.unidir) {
         sendpacket_close(options.sp2);
         pcap_close(options.listen2);
     }
