@@ -1,4 +1,4 @@
-/* $Id: tcpbridge.h 1795 2007-04-11 04:51:58Z aturner $ */
+/* $Id: tcpbridge.h 1944 2008-01-14 01:13:38Z aturner $ */
 
 /*
  * Copyright (c) 2005 Aaron Turner.
@@ -46,6 +46,11 @@
 #include <fcntl.h>
 #include <regex.h>
 
+#ifdef ENABLE_DMALLOC
+#include <dmalloc.h>
+#endif
+
+
 /* run-time options */
 struct tcpbridge_opt_s {
     char *intf1;
