@@ -1,4 +1,4 @@
-/* $Id: parse_args.c 1895 2007-08-22 04:24:33Z aturner $ */
+/* $Id: parse_args.c 1983 2008-04-25 04:51:07Z aturner $ */
 
 /*
  * Copyright (c) 2006 Aaron Turner.
@@ -51,7 +51,7 @@ int
 tcpedit_post_args(tcpedit_t **tcpedit_ex) {
     tcpedit_t *tcpedit;
     int rcode = 0;
-
+    long ttl;
     assert(tcpedit_ex);
     tcpedit = *tcpedit_ex;
     assert(tcpedit);
@@ -116,11 +116,29 @@ tcpedit_post_args(tcpedit_t **tcpedit_ex) {
     if (HAVE_OPT(FIXCSUM))
         tcpedit->fixcsum = 1;
 
-#ifdef ENABLE_EFCS
     /* --efcs */
     if (HAVE_OPT(EFCS)) 
         tcpedit->efcs = 1;
-#endif
+
+    /* --ttl */
+    if (HAVE_OPT(TTL)) {
+        if (strchr(OPT_ARG(TTL), '+')) {
+            tcpedit->ttl_mode = TCPEDIT_TTL_ADD;            
+        } else if (strchr(OPT_ARG(TTL), '-')) {
+            tcpedit->ttl_mode = TCPEDIT_TTL_SUB;            
+        } else {
+            tcpedit->ttl_mode = TCPEDIT_TTL_SET;            
+        }
+
+        ttl = strtol(OPT_ARG(TTL), (char **)NULL, 10);
+        if (ttl < 0)
+            ttl *= -1; /* convert to positive value */
+            
+        if (ttl > 255)
+            errx(1, "Invalid --ttl value (must be 0-255): %ld", ttl);
+
+        tcpedit->ttl_value = (u_int8_t)ttl;
+    }
 
     /* --mtu */
     if (HAVE_OPT(MTU))
