@@ -1,4 +1,4 @@
-/* $Id:$ */
+/* $Id: user.c 1893 2007-08-10 04:24:50Z aturner $ */
 
 /*
  * Copyright (c) 2006-2007 Aaron Turner.
@@ -41,7 +41,7 @@
 #include "tcpr.h"
 
 static char dlt_name[] = "user";
-static char __attribute__((unused)) dlt_prefix[] = "user";
+static char _U_ dlt_prefix[] = "user";
 static u_int16_t dlt_value = DLT_USER0;
 
 /*
@@ -88,7 +88,8 @@ dlt_user_register(tcpeditdlt_t *ctx)
     plugin->plugin_l2len = dlt_user_l2len;
     plugin->plugin_get_layer3 = dlt_user_get_layer3;
     plugin->plugin_merge_layer3 = dlt_user_merge_layer3;
-
+    plugin->plugin_get_mac = dlt_user_get_mac;
+    
     /* add it to the available plugin list */
     return tcpedit_dlt_addplugin(ctx, plugin);
 }
@@ -143,12 +144,12 @@ dlt_user_cleanup(tcpeditdlt_t *ctx)
 
     /* FIXME: make this function do something if necessary */
     if (ctx->decoded_extra != NULL) {
-        free(ctx->decoded_extra);
+        safe_free(ctx->decoded_extra);
         ctx->decoded_extra = NULL;
     }
         
     if (plugin->config != NULL) {
-        free(plugin->config);
+        safe_free(plugin->config);
         plugin->config = NULL;
     }
 
@@ -339,6 +340,20 @@ dlt_user_l2len(tcpeditdlt_t *ctx, const u_char *packet, const int pktlen)
     return config->length;
 }
 
+/*
+ * return a static pointer to the source/destination MAC address
+ * return NULL on error/address doesn't exist
+ */    
+u_char *
+dlt_user_get_mac(tcpeditdlt_t *ctx, _U_ tcpeditdlt_mac_type_t mac, const u_char *packet, const int pktlen)
+{
+    assert(ctx);
+    assert(packet);
+    assert(pktlen);
+
+    /* we don't know the format of USER DLT, hence always return NULL */
+    return(NULL);
+}
 
 tcpeditdlt_l2addr_type_t 
 dlt_user_l2addr_type(void)
