@@ -1,4 +1,4 @@
-/* $Id: tcpedit.c 1921 2007-10-25 18:18:50Z aturner $ */
+/* $Id: tcpedit.c 1983 2008-04-25 04:51:07Z aturner $ */
 
 /*
  * Copyright (c) 2001-2007 Aaron Turner.
@@ -108,8 +108,10 @@ tcpedit_packet(tcpedit_t *tcpedit, struct pcap_pkthdr **pkthdr,
      * only set this flag IFF the pcap has the FCS.  If not, then they
      * just removed 2 bytes of ACTUAL PACKET DATA.  Sucks to be them.
      */
-    if (tcpedit->efcs)
-        (*pkthdr)->caplen -= 2;
+    if (tcpedit->efcs > 0) {
+        (*pkthdr)->caplen -= 4;
+        (*pkthdr)->len -= 4;
+    }
         
     /* rewrite DLT */
     if ((pktlen = tcpedit_dlt_process(tcpedit->dlt_ctx, *pktdata, (*pkthdr)->caplen, direction)) == TCPEDIT_ERROR)
@@ -143,6 +145,9 @@ tcpedit_packet(tcpedit_t *tcpedit, struct pcap_pkthdr **pkthdr,
         ip_hdr = NULL;
     }
 
+    /* rewrite the TTL */
+    needtorecalc += rewrite_ipv4_ttl(tcpedit, ip_hdr);
+    
     /* rewrite IP addresses */
     if (tcpedit->rewrite_ip) {
         /* IP packets */
