@@ -1,7 +1,7 @@
-/* $Id: tcpreplay.c 1952 2008-01-17 05:43:20Z aturner $ */
+/* $Id: tcpreplay.c 2006 2008-05-01 04:46:55Z aturner $ */
 
 /*
- * Copyright (c) 2001-2007 Aaron Turner.
+ * Copyright (c) 2001-2008 Aaron Turner.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -67,6 +67,10 @@ int debug = 0;
 tcpedit_t *tcpedit;
 #endif
 
+#ifdef HAVE_ABSOLUTE_TIME
+#include <CoreServices/CoreServices.h>
+#endif
+
 void replay_file(int file_idx);
 void usage(void);
 void init(void);
@@ -275,6 +279,7 @@ post_args(void)
     char *temp, *intname;
     char ebuf[SENDPACKET_ERRBUF_SIZE];
     int int1dlt, int2dlt;
+    float gtod = 14.5;
     
 #ifdef ENABLE_PCAP_FINDALLDEVS
     interface_list_t *intlist = get_interface_list();
@@ -289,7 +294,7 @@ post_args(void)
     if (HAVE_OPT(DBUG))
         warn("not configured with --enable-debug.  Debugging disabled.");
 #endif
-    
+        
     options.loop = OPT_VALUE_LOOP;
 
     if (HAVE_OPT(LIMIT))
@@ -322,16 +327,57 @@ post_args(void)
 #endif
 
 	/*
-		Check if the file cache should be enabled - if we're looping more than
-		once and the command line option has been spec'd
-	*/
+	 * Check if the file cache should be enabled - if we're looping more than
+	 * once and the command line option has been spec'd
+	 */
 	if (HAVE_OPT(ENABLE_FILE_CACHE) && (options.loop != 1)) {
 		options.enable_file_cache = TRUE;
 	}
 
-    if (HAVE_OPT(ACCURATE))
-        options.accurate = 1;
-        
+    if (HAVE_OPT(TIMER)) {
+        if (strcmp(OPT_ARG(TIMER), "select") == 0) {
+#ifdef HAVE_SELECT
+            options.accurate = ACCURATE_SELECT;
+#else
+            err(1, "tcpreplay not compiled with select support");
+#endif
+        } else if (strcmp(OPT_ARG(TIMER), "rdtsc") == 0) {
+#ifdef HAVE_RDTSC
+            options.accurate = ACCURATE_RDTSC;
+#else
+            err(1, "tcpreplay not compiled with rdtsc support");
+#endif
+        } else if (strcmp(OPT_ARG(TIMER), "ioport") == 0) {
+#if defined HAVE_IOPERM && defined(__i386__)
+            options.accurate = ACCURATE_IOPORT;
+            ioport_sleep_init();
+#else
+            err(1, "tcpreplay not compiled with IO Port 0x80 support");
+#endif
+        } else if (strcmp(OPT_ARG(TIMER), "gtod") == 0) {
+            options.accurate = ACCURATE_GTOD;
+        } else if (strcmp(OPT_ARG(TIMER), "nano") == 0) {
+            options.accurate = ACCURATE_NANOSLEEP;
+        } else if (strcmp(OPT_ARG(TIMER), "abstime") == 0) {
+#ifdef HAVE_ABSOLUTE_TIME
+            options.accurate = ACCURATE_ABS_TIME;
+            if  (!MPLibraryIsLoaded()) {
+                err(1, "The MP library did not load.\n");
+            }            
+#else
+            err(1, "tcpreplay only supports absolute time on Apple OS X");
+#endif
+        } else {
+            errx(1, "Unsupported timer mode: %s", OPT_ARG(TIMER));
+        }
+    }
+
+#ifdef HAVE_RDTSC
+    if (HAVE_OPT(RDTSC_CLICKS)) {
+        rdtsc_calibrate(OPT_VALUE_RDTSC_CLICKS);
+    }
+#endif
+
     if (HAVE_OPT(PKTLEN))
         warn("--pktlen may cause problems.  Use with caution.");
     
