@@ -1,4 +1,4 @@
-/* $Id: tcprewrite.c 1898 2007-08-25 05:10:51Z aturner $ */
+/* $Id: tcprewrite.c 2001 2008-04-27 06:31:29Z aturner $ */
 
 /*
  * Copyright (c) 2004-2007 Aaron Turner.
@@ -77,7 +77,9 @@ main(int argc, char *argv[])
 {
     int optct, rcode;
     pcap_t *dlt_pcap;
-    
+#ifdef ENABLE_FRAGROUTE
+    char ebuf[FRAGROUTE_ERRBUF_LEN];
+#endif
     tcprewrite_init();
 
     /* call autoopts to process arguments */
@@ -106,7 +108,7 @@ main(int argc, char *argv[])
         errx(1, "Unable to edit packets given options:\n%s",
                 tcpedit_geterr(tcpedit));
     }
-
+    
    /* open up the output file */
     options.outfile = safe_strdup(OPT_ARG(OUTFILE));
     dbgx(1, "Rewriting DLT to %s",
@@ -117,6 +119,13 @@ main(int argc, char *argv[])
     dbgx(1, "DLT of dlt_pcap is %s",
         pcap_datalink_val_to_name(pcap_datalink(dlt_pcap)));
 
+#ifdef ENABLE_FRAGROUTE
+    if (options.fragroute_args) {
+        if ((options.frag_ctx = fragroute_init(65535, pcap_datalink(dlt_pcap), options.fragroute_args, ebuf)) == NULL)
+            errx(1, "%s", ebuf);
+    }
+#endif
+
 #ifdef ENABLE_VERBOSE
     if (options.verbose) {
         tcpdump_open(&tcpdump, dlt_pcap);
@@ -140,6 +149,9 @@ main(int argc, char *argv[])
     tcpdump_close(&tcpdump);
 #endif
 
+#ifdef ENABLE_DMALLOC
+    dmalloc_shutdown();
+#endif
     return 0;
 }
 
@@ -183,6 +195,25 @@ post_args(_U_ int argc, _U_ char *argv[])
         tcpdump.args = safe_strdup(OPT_ARG(DECODE));    
 #endif
 
+
+#ifdef ENABLE_FRAGROUTE
+    if (HAVE_OPT(FRAGROUTE))
+        options.fragroute_args = safe_strdup(OPT_ARG(FRAGROUTE));
+    
+    options.fragroute_dir = FRAGROUTE_DIR_BOTH;
+    if (HAVE_OPT(FRAGDIR)) {
+        if (strcmp(OPT_ARG(FRAGDIR), "c2s") == 0) {
+            options.fragroute_dir = FRAGROUTE_DIR_C2S;
+        } else if (strcmp(OPT_ARG(FRAGDIR), "s2c") == 0) {
+            options.fragroute_dir = FRAGROUTE_DIR_S2C;
+        } else if (strcmp(OPT_ARG(FRAGDIR), "both") == 0) {
+            options.fragroute_dir = FRAGROUTE_DIR_BOTH;
+        } else {
+            errx(1, "Unknown --fragdir value: %s", OPT_ARG(FRAGDIR));
+        }
+    }
+#endif
+
     /* open up the input file */
     options.infile = safe_strdup(OPT_ARG(INFILE));
     if ((options.pin = pcap_open_offline(options.infile, ebuf)) == NULL)
@@ -199,11 +230,15 @@ rewrite_packets(tcpedit_t *tcpedit, pcap_t *pin, pcap_dumper_t *pout)
     struct pcap_pkthdr pkthdr, *pkthdr_ptr;     /* packet header */
     const u_char *pktdata = NULL;               /* packet from libpcap */
     u_char **packet = NULL;                     /* packet from tcpedit */
+    static char *frag = NULL;
     COUNTER packetnum = 0;
-    int rcode;
+    int rcode, frag_len, i;
 
     pkthdr_ptr = &pkthdr;
 
+    if (frag == NULL)
+        frag = (char *)safe_malloc(65535); /* mtu size */
+
     /* MAIN LOOP 
      * Keep sending while we have packets or until
      * we've sent enough packets
@@ -212,6 +247,8 @@ rewrite_packets(tcpedit_t *tcpedit, pcap_t *pin, pcap_dumper_t *pout)
         packetnum++;
         dbgx(2, "packet " COUNTER_SPEC " caplen %d", packetnum, pkthdr.caplen);
 
+        packet = (u_char **)&pktdata;
+
 #ifdef ENABLE_VERBOSE
         if (options.verbose)
             tcpdump_print(&tcpdump, pkthdr_ptr, pktdata);
@@ -231,7 +268,6 @@ rewrite_packets(tcpedit_t *tcpedit, pcap_t *pin, pcap_dumper_t *pout)
         if (cache_result == TCPR_DIR_NOSEND)
             goto WRITE_PACKET; /* still need to write it so cache stays in sync */
 
-        packet = &pktdata;
         if ((rcode = tcpedit_packet(tcpedit, &pkthdr_ptr, packet, cache_result)) == TCPEDIT_ERROR) {
             return -1;
         } else if ((rcode == TCPEDIT_SOFT_ERROR) && HAVE_OPT(SKIP_SOFT_ERRORS)) {
@@ -242,9 +278,37 @@ rewrite_packets(tcpedit_t *tcpedit, pcap_t *pin, pcap_dumper_t *pout)
 
 
 WRITE_PACKET:
-        /* write the packet */
-        pcap_dump((u_char *)pout, pkthdr_ptr, *packet);
+#ifdef ENABLE_FRAGROUTE
+        if (options.frag_ctx == NULL) {
+            /* write the packet when there's no fragrouting to be done */
+            pcap_dump((u_char *)pout, pkthdr_ptr, *packet);
+        } else {
+            /* packet needs to be fragmented */
+            if ((options.fragroute_dir == FRAGROUTE_DIR_BOTH) ||
+                    (cache_result == TCPR_DIR_C2S && options.fragroute_dir == FRAGROUTE_DIR_C2S) ||
+                    (cache_result == TCPR_DIR_S2C && options.fragroute_dir == FRAGROUTE_DIR_S2C)) {
 
+                if (fragroute_process(options.frag_ctx, *packet, pkthdr_ptr->caplen) < 0)
+                    errx(1, "Error processing packet via fragroute: %s", options.frag_ctx->errbuf);
+
+                i = 0;
+                while ((frag_len = fragroute_getfragment(options.frag_ctx, &frag)) > 0) {
+                    /* frags get the same timestamp as the original packet */
+                    dbgx(1, "processing packet " COUNTER_SPEC " frag: %u (%d)", packetnum, i++, frag_len);
+                    pkthdr_ptr->caplen = frag_len;
+                    pkthdr_ptr->len = frag_len;
+                    pcap_dump((u_char *)pout, pkthdr_ptr, (u_char *)frag);
+                }
+            } else {
+                /* write the packet without fragroute */
+                pcap_dump((u_char *)pout, pkthdr_ptr, *packet);
+            }
+        }
+#else
+    /* write the packet when there's no fragrouting to be done */
+    pcap_dump((u_char *)pout, pkthdr_ptr, *packet);
+
+#endif
     } /* while() */
     return 0;
 }   
