@@ -1,4 +1,4 @@
-/* $Id: tcprewrite.h 1795 2007-04-11 04:51:58Z aturner $ */
+/* $Id: tcprewrite.h 1950 2008-01-14 18:09:59Z aturner $ */
 
 /*
  * Copyright (c) 2004-2007 Aaron Turner.
@@ -38,6 +38,14 @@
 #include "defines.h"
 #include "tcpedit/tcpedit.h"
 
+#ifdef ENABLE_DMALLOC
+#include <dmalloc.h>
+#endif
+
+#ifdef ENABLE_FRAGROUTE
+#include "fragroute/fragroute.h"
+#endif
+
 /* runtime options */
 struct tcprewrite_opt_s {
     /* input and output pcap filenames & handles */
@@ -58,6 +66,15 @@ struct tcprewrite_opt_s {
     int verbose;
     char *tcpdump_args;
 #endif
+
+#ifdef ENABLE_FRAGROUTE
+    char *fragroute_args;
+    fragroute_t *frag_ctx;
+#define FRAGROUTE_DIR_C2S  1
+#define FRAGROUTE_DIR_S2C  2
+#define FRAGROUTE_DIR_BOTH 4
+    int fragroute_dir;
+#endif
     tcpedit_t *tcpedit;
 };
 
